<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:html="http://www.w3.org/1999/xhtml"
    xmlns="http://www.w3.org/1999/xhtml"
    exclude-result-prefixes="html">

    <xsl:output
        method="xml"
        doctype-system="http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"
        doctype-public="-//W3C//DTD XHTML 1.1//EN"
    	indent="yes"
    	encoding="UTF-8"
    />

	<xsl:template match="/">
		<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="de">
			<head>
				<title>Database export</title>
				<link rel="stylesheet" href="default.css" type="text/css" />
			</head>
			<body>
				<xsl:apply-templates/>
			</body>
		</html>
	</xsl:template>
	
	<xsl:template match="/database">
		<h1>database: <xsl:value-of select="@name"/></h1>
		<h2>created: <xsl:value-of select="created/text()"/></h2>
		<ul>
			<xsl:for-each select="table">
				<li>
					<a>
						<xsl:attribute name="href"><xsl:value-of select="text()"/>.xml</xsl:attribute>
						table <xsl:value-of select="text()"/>
					</a>
				</li>
			</xsl:for-each>
		</ul>
	</xsl:template>
	
	<xsl:template match="/table">
		<h1>table: <xsl:value-of select="@name"/></h1>
		<h2>created: <xsl:value-of select="created/text()"/></h2>
		<dl>
			<xsl:for-each select="field">
				<dt><xsl:value-of select="@name"/></dt>
				<dd>
					<ul>
						<li>
							autoincrement: 
							<xsl:choose>
								<xsl:when test="@autoincrement = 'yes'">yes</xsl:when>
								<xsl:otherwise>no</xsl:otherwise>
							</xsl:choose>
						</li>
						<li>
							<xsl:value-of select="type/@name"/>
							<xsl:choose>
								<xsl:when test="type/@unsigned/text() = 'yes'"> (unsigned)</xsl:when>
								<xsl:otherwise></xsl:otherwise>
							</xsl:choose>
						</li>
						<xsl:if test="default">
							<li>
								default: <xsl:value-of select="default/text()"/>
							</li>
						</xsl:if>
						<li>
							is primary key: 
							<xsl:variable name="fieldName" select="@name"/>
							<xsl:choose>
								<xsl:when test="/table/constraints/key[@type = 'primary'] = $fieldName">yes</xsl:when>
								<xsl:otherwise>no</xsl:otherwise>
							</xsl:choose>
						</li>
						<li>
							is index key: 
							<xsl:variable name="fieldName" select="@name"/>
							<xsl:choose>
								<xsl:when test="/table/constraints/key[@type = 'index'] = $fieldName">yes</xsl:when>
								<xsl:otherwise>no</xsl:otherwise>
							</xsl:choose>
						</li>
					</ul>
				</dd>
			</xsl:for-each>
		</dl>
	</xsl:template>
	
	<xsl:template match="/structure">
		<h1>database: <xsl:value-of select="database/@name"/> (created: <xsl:value-of select="created/text()"/>)</h1>
		<p>
			The database <em><xsl:value-of select="database/@name"/></em> contains <xsl:value-of select="count(database/table)"/> tables.
		</p>
		<h2>tables</h2>
		<xsl:for-each select="table">
			<xsl:call-template name="content-table">
				<xsl:with-param name="table" select="."/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="content-table">
		<xsl:param name="table"/>
		<xsl:variable name="tableName" select="$table/@name" />
		
		<h3><xsl:value-of select="$tableName"/></h3>
		<h4>overview</h4>
		<table>
			<thead>
				<tr>
					<th>field</th>
					<th>type</th>
					<th>default</th>
					<th>autoincrement</th>
					<th>unsigned</th>
					<th>index</th>
					<th>foreign key on..</th>
					<th>sub keys..</th>
				</tr>
			</thead>
			<tbody>
				<xsl:for-each select="field">
					<xsl:variable name="fieldName" select="@name"/>
					<tr>
						<xsl:attribute name="class">
							<xsl:if test="position() mod 2">alternate </xsl:if>
							<xsl:choose>
								<xsl:when test="$table/constraints/field[@type = 'primary' and text() = $fieldName]">primaryKey</xsl:when>
								<xsl:when test="$table/constraints/field[@type = 'index' and text() = $fieldName]">indexKey</xsl:when>
							</xsl:choose>
						</xsl:attribute>
						<td class="table-field-name"><xsl:value-of select="$fieldName"/></td>
						<td><xsl:value-of select="type/@name"/></td>
						<td><xsl:if test="default"><xsl:value-of select="default/text()"/></xsl:if><xsl:text> </xsl:text></td>
						<td><xsl:call-template name="yesno" select="@autoincrement"/></td>
						<td><xsl:call-template name="yesno" select="type/@unsigned"/></td>
						<td>
							<xsl:if test="count($table/constraints/field[@type = 'primary' and text() = $fieldName]) &gt; 0">primary</xsl:if>
							<xsl:if test="count($table/constraints/field[@type = 'index' and text() = $fieldName]) &gt; 0">index</xsl:if>
							<xsl:text> </xsl:text>
						</td>
						<td>
							<xsl:call-template name="list-foreign-key">
								<xsl:with-param name="tableName" select="$tableName"/>
								<xsl:with-param name="fieldName" select="$fieldName"/>
							</xsl:call-template>
						</td>
						<td>
							<xsl:if test="count($table/constraints/field[@type = 'primary' and text() = $fieldName]) &gt; 0">
								<xsl:call-template name="list-sub-keys">
									<xsl:with-param name="tableName" select="$tableName"/>
									<xsl:with-param name="tables" select="/structure/database/references/field[@name = $fieldName]"/>
								</xsl:call-template>
							</xsl:if>
						</td>
					</tr>
				</xsl:for-each>
			</tbody>
		</table>
		
		<h4>messages</h4>
		<ul>
			<xsl:choose>
				<xsl:when test="count(/structure/messages/message[@table = $tableName]) &gt; 0">
					<xsl:for-each select="/structure/messages/message[@table = $tableName]">
						<li>
							<xsl:value-of select="@field"/> has <xsl:apply-templates mode="content" select="text"/>
						</li>
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>
					<li>none</li>
				</xsl:otherwise>
			</xsl:choose>
		</ul>
	</xsl:template>
	
	<xsl:template name="yesno">
		<xsl:choose>
			<xsl:when test="text() = 'yes'">yes</xsl:when>
			<xsl:otherwise>no</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="list-foreign-key">
		<xsl:param name="tableName"/>
		<xsl:param name="fieldName"/>
		
		<xsl:choose>
			<xsl:when test="count(/structure/database/references/field[@name = $fieldName]/table[@type = 'master' and not(text() = $tableName)]) &gt; 0">
				<!-- <ul>
					<xsl:for-each select="/structure/database/references/field[@name = $fieldName]/table[@type = 'master' and not(text() = $tableName)]">
						<li><xsl:value-of select="text()"/></li>
					</xsl:for-each>
				</ul>-->
				<xsl:call-template name="list-seperated-by-delimiter">
					<xsl:with-param name="items" select="/structure/database/references/field[@name = $fieldName]/table[@type = 'master' and not(text() = $tableName)]"/>
					<xsl:with-param name="delimiter">, </xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				no found
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- only table names -->
	<xsl:template name="list-sub-keys">
		<xsl:param name="tableName"/>
		<xsl:param name="tables"/>
		
		<xsl:choose>
			<xsl:when test="count($tables/table[@type = 'detail']) &gt; 0">
				<!-- <ul>
					<xsl:for-each select="$tables/table[@type = 'detail']">
						<li><xsl:value-of select="text()"/></li>
					</xsl:for-each>
				</ul>-->
				<xsl:call-template name="list-seperated-by-delimiter">
					<xsl:with-param name="items" select="$tables/table[@type = 'detail']"/>
					<xsl:with-param name="delimiter">, </xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				no found
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- only table names -->
	<xsl:template name="list-seperated-by-delimiter">
		<xsl:param name="items"/>
		<xsl:param name="delimiter"/>
		
		<xsl:for-each select="$items">
			<xsl:variable name="x" select="."/>
			<a><xsl:attribute name="href"><xsl:value-of select="$x"/>.xml</xsl:attribute><xsl:value-of select="$x"/></a>
			<xsl:if test="not(position() = last())"><xsl:value-of select="$delimiter"/></xsl:if>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="text()" mode="content">
		<xsl:value-of select="."/>
	</xsl:template>
	
	<xsl:template match="code" mode="content">
		<code>
			<xsl:apply-templates mode="content"/>
		</code>
	</xsl:template>
</xsl:stylesheet>